import numpy as np
import matplotlib.pyplot as plt
import matplotlib
import astropy.units as u
import matplotlib.patches as mpatches
import matplotlib.lines as mlines
import pickle
import pandas as pd
from xraydlps.tools import load, convert

cs = ['grbs', 'sbo', 'sne', 'tdes', 'agn', 
			'fbots', 'novae', 'dne', 'magnetars', 
			'frb', 'coolstellar', 'xrbs', 'ulxs']

sc = [['long', 'short', 'ultralong', 'subluminous'], None, 
			['cc-I', 'cc-II', 'interacting', 'Ca-rich', 'superluminous'], 
			['nonthermal', 'thermal'], None, None, None, None, 
			['outburst', 'flare'], None, None, ['highmass', 'lowmass'], None]

defaultc_colors = ['#2ca02c', '#17becf', '#d62728', '#ff7f0e', 
		'#cc5f00', '#8c564b', 'xkcd:burnt yellow', 'xkcd:wheat', '#e377c2', 
		'gray', '#9467bd', '#1f77b4', '#004481'] #defined for major classes of transient

defaultsc_colors = [['#2ca02c', '#79ED79', '#1f701f', '#124012'], '#1298a6', 
		['#d62728', '#ff8b8c', '#8A0000', '#400b0c', '#f2bebe'], 
		['#ff7f0e', '#FFB241'], '#cc5f00', '#8c564b',  'xkcd:burnt yellow', 'xkcd:wheat', 
		['#e377c2', '#FFAAF5'], 'gray', '#9467bd', ['#1f77b4', '#52AAE7'], 
		'#004481'] #defined for subclasses of transient


def list_classes(subs = False, colors = False):
	"""
	List available classes of transients.

	Parameters:
		subs (bool): If subs = True, return tuples with classes + subclasses of transients.
		colors (bool): If colors = True, return tuple that includes default plotting colors.
	"""
	if not subs:
		if not colors:
			for c in cs:
				print(c)
		if colors:
			for z in zip(cs, defaultc_colors):
				print(z)
	if subs:
		if not colors:
			for z in zip(cs, sc):
				print(z)
		if colors:
			for z in zip(cs, sc, defaultsc_colors):
				print(z)

##########
# * * * * 
##########

def set_mpldefaults(fontfamily = 'serif',
					fontsize = 45,
					majortick_width = 3, 
					majortick_size = 15,
					minorticks = False,
					minortick_width = 3,
					minortick_size = 10,
					rightticks = True,
					hatch_width = 3,
					linewidth = 8,
					markersize = 20):
	"""
	Set matplotlib.rc defaults for DLPS.

	Not necessary for plots, but uses style Polzin et al. (2023).

	Parameters:
		fontfamily (str): Font family to use.
		fontsize (int): Fontsize to use.
		majortick_width (int): Width of major ticks.
		majortick_size (int): Length of major ticks.
		minorticks (bool): If True, include minor ticks.
		minortick_width (int): Width of minor ticks.
		minortick_size (int): Length of minor ticks.
		rightticks (bool): If True, include right ticks.
		hatch_width (int): Width of hatches.
		linewidth (int): Default line width.
		markersize (int): Default marker size.
	"""
	matplotlib.rcParams['font.family'] = fontfamily
	# matplotlib.rcParams['font.size'] = fontsize
	matplotlib.rcParams['xtick.major.size'] = majortick_size
	matplotlib.rcParams['ytick.major.size'] = majortick_size
	matplotlib.rcParams['xtick.major.width'] = majortick_width
	matplotlib.rcParams['ytick.major.width'] = majortick_width
	matplotlib.rcParams['xtick.minor.visible'] = minorticks
	matplotlib.rcParams['xtick.minor.visible'] = minorticks
	if minorticks:
		matplotlib.rcParams['xtick.minor.size'] = minortick_size
		matplotlib.rcParams['ytick.minor.size'] = minortick_size
		matplotlib.rcParams['xtick.minor.width'] = minortick_width
		matplotlib.rcParams['ytick.minor.width'] = minortick_width
	matplotlib.rcParams['ytick.right'] = rightticks
	matplotlib.rc('hatch', linewidth = hatch_width)
	matplotlib.rcParams['lines.linewidth'] = linewidth
	matplotlib.rcParams['lines.markersize'] = markersize
	matplotlib.rc('font', size = fontsize)






def dlps_axes(xlabel = 'Rest Frame Time Since Identification (days)', 
			  ylabel = r'0.3 - 10 keV X-ray Luminosity (erg s$^{-1}$)', 
			  add_minorxticks = [10.**i for i in np.arange(-8, 4)],
			  add_minoryticks = [10.**i for i in np.arange(26, 51)],
			  ax = plt, xlim = [3e-9, 5e4], ylim = [1e26, 1e51],
			  rightticks = True, tickwidth = [3, 3],
			  xscale = 'log', yscale = 'log'):
	"""
	Quickly set up axes for X-ray DLPS plots. For most full DLPS plots, should not need to change defaults.

	Parameters:
		xlabel (str): Label for x-axis of DLPS plot.
		ylabel (str): Label for y-axis of DLPS plot.
		add_minorxticks (arr-like): If None, will not add ticks. Otherwise, list of x-values for which to add minor tick.
		add_minoryticks (arr-like): If None, will not add ticks. Otherwise, list of y-values for which to add minor tick.
		ax (axis object): Axis on which to add ticks/labels; e.g., set plt.xlabel or ax.set_xlabel.
		xlim (arr-like): Lower and upper limit of x-axis.
		ylim (arr-like): Lower and upper limit of y-axis.
		rightticks (bool): If True, plot minor ticks on right y-axis. 
							(Looks best if major ticks are enabled for the rightside, too.)
		tickwidth (list): Width of major and minor ticks to be added respectively. Best if this matches set_mpldefaults(). 
		xscale (str): Set scaling of x-axis.
		yscale (str): Set scaling of y-axis.

	(Can hard code changes to minor tick lengths to suit personal preference.)
	"""

	xtick_len = (ylim[1] - ylim[0])/(1e51 - 1e26) * (1e26 - 6.5e25) #can hard code changes to suit preferences
	ytick_len_left = (xlim[1] - xlim[0])/(5e4 - 3e-9) * (3e-9 - 2.6e-9)
	ytick_len_right = (xlim[1] - xlim[0])/(5e4 - 3e-9) * (6e4 - 5e4)

	if add_minorxticks:
		for i in add_minorxticks:
			ax.vlines(x = i, ymin = ylim[0] - xtick_len, ymax = ylim[0], clip_on = False, color = 'k', lw = tickwidth[1])
		for j in add_minoryticks:
			ax.vlines(x = i, ymin = xlim[0] - ytick_len_left, ymax = xlim[0], clip_on = False, color = 'k', lw = tickwidth[1])
			if rightticks:
				ax.vlines(x = i, ymin = xlim[1], ymax = xlim[1] + ytick_len_right, clip_on = False, color = 'k', lw = tickwidth[0])

	if ax == plt:
		ax.xlabel(xlabel)
		ax.ylabel(ylabel)
		ax.xlim(xlim)
		ax.ylim(ylim)
		ax.xscale(xscale)
		ax.yscale(yscale)

	else:
		ax.set_xlabel(xlabel)
		ax.set_ylabel(ylabel)
		ax.set_xlim(xlim)
		ax.set_ylim(ylim)
		ax.set_xscale(xscale)
		ax.set_yscale(yscale)


def dlps_legend(labels = [cs, sc], colors = [defaultc_colors, defaultsc_colors], 
				ax = plt, style = 'patch', marker = None, subson = False, **kwargs):
	"""
	Easily enerate custom legend for plots.

	Parameters:
		labels (arr-like): List of strings that serve as labels in legend.
		colors (arr-like): List of colors corresponding to labels.
		ax (axis object): Axis on which to add legend, e.g., ax.legend().
		style (str or arr-like): 'Patch', 'line', or 'scatter' -- 
				if 'scatter', have to specify marker. Case insensitive.
		marker (str or arr-like): Marker to be used in legend.
		subson (bool): Default is False. If true, use subclass legend.
		**kwargs: Modify ax.legend(), see matplotlib documentation.


	"""
	if not subson:
		labels = labels[0]
		colors = colors[0]
	if subson:
		labels_ = []
		for i, s in enumerate(sc):
			if s:
				for s_ in s:
					if cs[i] == 'magnetars':
						labels_.append(s_+', '+cs[i])
					else:
						labels_.append(s_+' '+cs[i])
			if not s:
				labels_.append(cs[i])
			
		colors_ = []
		for c in defaultsc_colors:
			if type(c) == list:
				for i in c:
					colors_.append(i)
			if type(c) == str:
				colors_.append(c)
		labels = labels_
		colors = colors_


	if type(style) == str:
		style = [style]*len(labels)
	if type(marker) == str:
		marker = [marker]*len(labels)
	handles = []
	for i, l in enumerate(labels):
		if style[i].lower() == 'patch':
			handles.append(mpatches.Patch(color = colors[i], label = l))
		if style[i].lower() == 'line':
			handles.append(mlines.Line2D([], [], color = colors[i], label = l))
		if style[i].lower() == 'scatter':
			handles.append(mlines.Line2D([], [], color = colors[i], label = l, 
				linestyle = '', marker = 'o'))

	ax.legend(handles = handles, **kwargs)

	
	
def obs_dlps(classes = cs, subclasses = sc, subson = False,
			 colors = [defaultc_colors, defaultsc_colors], 
			 xunit = u.d, yunit = u.erg/u.s, k = 1., ax = plt, **kwargs):

	"""
	Plot the observational DLPS.

	Parameters:
		classes (arr-like): List of classes to plot.
		subclasses (arr-like): If subson = True, plot these subclasses. Should be a nested list, 
			with subclasses grouped in sub-list by class.
		subson (bool): Plot subclasses instead of classes -- if subson = True, must specify subclasses.
		xunit (astropy units object): Units of time for observational DLPS. Default is days.
		yunit (astropy units object): Units of luminosity for observational DLPS. Default is erg/s
		k (float or arr-like): Value of k-correction if plotting outside of 0.3-10 keV band. 
			Should be the length of classes.
		ax (axis object): Axis on which to plot, e.g., ax.plot().
		**kwargs: Modify ax.plot(), see matplotlib documentation.
	"""
	lcs = load()

	xconv = u.d.to(xunit)
	yconv = (u.erg/u.s).to(yunit)
	if (type(k) == float) or (type(k) == int):
		k = [k]*len(classes)

	if colors == [defaultc_colors, defaultsc_colors]: # hacky test for default parameters (might be temporary if I find a better way)
		if subson:
			colors = colors[1]
		if not subson:
			colors = colors[0]

	if subson:
		for i, c in enumerate(classes):
			ki = k[i]
			if subclasses[i]:
				if sc[i]:
					for k_, s in enumerate(sc[i]):
						col = colors[i][k_]
						for j, obj in enumerate(lcs.loc[(lcs['class'] == c) & (lcs['subclass'] == s)].iterrows()):
							# for obj in lcs.loc[(lcs['class'] == c) & (lcs['subclass'] == s)]:
							time = obj[1]['time'] * xconv
							lum = obj[1]['lum'] * yconv * ki
							ax.plot(time, lum, color = col)

							if len(time) == 1:
								ax.scatter(time, lum, color = col) #s = lw^2

			if not subclasses[i]:
				col = colors[i]
				for j, obj in enumerate(lcs.loc[lcs['class'].values == c].iterrows()):
					time = obj[1]['time'] * xconv
					lum = obj[1]['lum'] * yconv * ki
					ax.plot(time, lum, color = col)

					if len(time) == 1:
						ax.scatter(time, lum, color = col)

	if not subson:
		for i, c in enumerate(classes):
			ki = k[i]
			col = colors[i]
			for j, obj in enumerate(lcs.loc[lcs['class'].values == c].iterrows()):
				time = obj[1]['time'] * xconv
				lum = obj[1]['lum'] * yconv * ki
				ax.plot(time, lum, color = col)

				if len(time) == 1:
					ax.scatter(time, lum, color = col)




def schematic_dlps(classes = np.concatenate((cs, ['sgrbs'])), expecton = False,
				colors = np.concatenate((defaultc_colors, ['#79ED79'])), k = 1, ax = plt,
				xunit = u.d, yunit = u.erg/u.s, **kwargs):

	"""
	Plot the schematic DLPS.

	Parameters:
		classes (arr-like): List of classes to plot. In addition to other classes, 'sgrbs' is a valid option.
		expecton (bool): Plot region where we would expect SBOs + magnetars (if included in classes).
		xunit (astropy units object): Units of time for observational DLPS. Default is days.
		yunit (astropy units object): Units of luminosity for observational DFPS. Default is erg/s
		k (float or arr-like): Value of k-correction if plotting outside of 0.3-10 keV band. 
			Should be the length of classes.
		ax (axis object): Axis on which to plot, e.g., ax.plot().
		**kwargs: Modify ax.plot(), see matplotlib documentation.
	"""
	carr = np.array(classes)
	colarr = np.array(colors)
	if (type(k) == float) or (type(k) == int):
		k = [k]*len(classes)
	karr = np.array(k)

	xconv = u.d.to(xunit)
	yconv = (u.erg/u.s).to(yunit)

	if expecton:
		if 'sbo' in classes:
			plt.fill_between([3e-5*xconv, 1*xconv], 
				[1e40*yconv*karr[carr == 'sbo'][0], 1e35*yconv*karr[carr == 'sbo'][0]], 
				[1e49*yconv*karr[carr == 'sbo'][0], 1e44*yconv*karr[carr == 'sbo'][0]], 
				edgecolor = colarr[carr == 'sbo'][0], facecolor = 'none', hatch = 'X', 
				linewidth = 0) #SS SBO

		if 'magnetars' in classes:
			plt.fill_between([10**-5.3*xconv, 10**0.3*xconv], 
				[10**41.5*yconv*karr[carr == 'magnetars'][0], 10**36.5*yconv*karr[carr == 'magnetars'][0]], 
				[10**38.5*yconv*karr[carr == 'magnetars'][0], 10**33.5*yconv*karr[carr == 'magnetars'][0]], 
				edgecolor = colarr[carr == 'magnetars'][0], hatch = '+', facecolor = 'none', 
				linewidth = 0) #magnetar exp

	if 'agn' in classes:
		plt.fill_between([8e-2*xconv, 5e3*xconv], 
			[2e38*yconv*karr[carr == 'agn'][0], 2e38*yconv*karr[carr == 'agn'][0]], 
			[3e46*yconv*karr[carr == 'agn'][0], 3e46*yconv*karr[carr == 'agn'][0]], 
			color = colarr[carr == 'agn'][0]) #AGN - 1
		plt.fill_between([2.5e-3*xconv, 8e-2*xconv], 
			[1e41*yconv*karr[carr == 'agn'][0]], 
			[3e42*yconv*karr[carr == 'agn'][0], 3e42*yconv*karr[carr == 'agn'][0]], 
			color = colarr[carr == 'agn'][0]) #AGN - 2, QPEs

	if 'grbs' in classes:
		plt.fill_between([3e-4*xconv, 2e3*xconv], 
			[1e46*yconv*karr[carr == 'grbs'][0], 3e38*yconv*karr[carr == 'grbs'][0]], 
			[6e50*yconv*karr[carr == 'grbs'][0], 3e43*yconv*karr[carr == 'grbs'][0]], 
			color = colarr[carr == 'grbs'][0]) #GRBs - 1
		plt.fill_between([5e-5*xconv, 3*xconv], 
			[3e45*yconv*karr[carr == 'grbs'][0], 1e40*yconv*karr[carr == 'grbs'][0]], 
			[3e47*yconv*karr[carr == 'grbs'][0], 1e42*yconv*karr[carr == 'grbs'][0]], 
			color = colarr[carr == 'grbs'][0]) #GRBs - 2
		if 'sgrbs' in classes:
			scol = colarr[carr == 'sgrbs'][0]
			sk = karr[carr == 'sgrbs'][0]
		if 'sgrbs' not in classes:
			scol = colarr[carr == 'grbs'][0]
			sk = karr[carr == 'grbs'][0]
		plt.fill_between([3e-4*xconv, 7e2*xconv], 
			[4.2e44*yconv*sk, 3e36*yconv*sk], 
			[7e49*yconv*sk, 5e41*yconv*sk], 
			color = scol)#sGRBs

	if 'tdes' in classes:
		plt.fill_between([1e-2*xconv, 1e4*xconv], 
			[3e46*yconv*karr[carr == 'tdes'][0], 3e42*yconv*karr[carr == 'tdes'][0]], 
			[1e50*yconv*karr[carr == 'tdes'][0], 1e46*yconv*karr[carr == 'tdes'][0]], 
			color = colarr[carr == 'tdes'][0]) #TDEs - 1
		plt.fill_between([3e0*xconv, 1e4*xconv], 
			[3e37*yconv*karr[carr == 'tdes'][0], 3e37*yconv*karr[carr == 'tdes'][0]], 
			[1e45*yconv*karr[carr == 'tdes'][0], 1e45*yconv*karr[carr == 'tdes'][0]], 
			color = colarr[carr == 'tdes'][0]) #TDEs - 2

	if 'sne' in classes:
		plt.fill_between([2.5e-1*xconv, 1.5e4*xconv], 
			[1e39*yconv*karr[carr == 'sne'][0], 1e34*yconv*karr[carr == 'sne'][0]], 
			[1e42*yconv*karr[carr == 'sne'][0], 1e42*yconv*karr[carr == 'sne'][0]], 
			color = colarr[carr == 'sne'][0])

	if 'sbo' in classes:
		plt.fill_between([2e-4*xconv, 1e-2*xconv], 
			[1e43*yconv*karr[carr == 'sbo'][0], 2e41*yconv*karr[carr == 'sbo'][0]], 
			[5e44*yconv*karr[carr == 'sbo'][0], 1e43*yconv*karr[carr == 'sbo'][0]], 
			color = colarr[carr == 'sbo'][0]) #wind SBO

	if 'xrbs' in classes:
		plt.fill_between([1e-6*xconv, 3e3*xconv], 
			[1e31*yconv*karr[carr == 'xrbs'][0], 1e31*yconv*karr[carr == 'xrbs'][0]], 
			[2e35*yconv*karr[carr == 'xrbs'][0], 2e35*yconv*karr[carr == 'xrbs'][0]], 
			color = colarr[carr == 'xrbs'][0]) #XRBs - 1
		plt.fill_between([1.5e-1*xconv, 3e3*xconv], 
			[2e35*yconv*karr[carr == 'xrbs'][0], 2e35*yconv*karr[carr == 'xrbs'][0]], 
			[3e39*yconv*karr[carr == 'xrbs'][0], 3e39*yconv*karr[carr == 'xrbs'][0]], 
			color = colarr[carr == 'xrbs'][0]) #XRBs - 2

	if 'dne' in classes:
		plt.fill_between([7e-4*xconv, 2e3*xconv], 
			[4e29*yconv*karr[carr == 'dne'][0], 4e29*yconv*karr[carr == 'dne'][0]], 
			[1e34*yconv*karr[carr == 'dne'][0], 1e34*yconv*karr[carr == 'dne'][0]], 
			color = colarr[carr == 'dne'][0]) #DNe

	if 'novae' in classes:
		plt.fill_between([1.5e-1*xconv, 6e3*xconv], 
			[1.5e30*yconv*karr[carr == 'novae'][0], 1.5e30*yconv*karr[carr == 'novae'][0]], 
			[7e36*yconv*karr[carr == 'novae'][0], 7e36*yconv*karr[carr == 'novae'][0]], 
			color = colarr[carr == 'novae'][0]) #Novae

	if 'coolstellar' in classes:
		plt.fill_between([2e-3*xconv, 1.2e0*xconv], 
			[1.5e28*yconv*karr[carr == 'coolstellar'][0], 1.5e28*yconv*karr[carr == 'coolstellar'][0]], 
			[6e32*yconv*karr[carr == 'coolstellar'][0], 6e32*yconv*karr[carr == 'coolstellar'][0]], 
			color = colarr[carr == 'coolstellar'][0]) #cool stellar flares

	if 'frb' in classes:
		plt.fill_between([2.5e-8*xconv, 1.5e-5*xconv], 
			[5e39*yconv*karr[carr == 'frb'][0], 5e39*yconv*karr[carr == 'frb'][0]], 
			[1e42*yconv*karr[carr == 'frb'][0], 1e42*yconv*karr[carr == 'frb'][0]], 
			color = colarr[carr == 'frb'][0]) #FRB

	if 'magnetars' in classes:
		plt.fill_between([1e-6*xconv, 3e-5*xconv], 
			[2e39*yconv*karr[carr == 'magnetars'][0], 2e39*yconv*karr[carr == 'magnetars'][0]], 
			[9e40*yconv*karr[carr == 'magnetars'][0], 9e40*yconv*karr[carr == 'magnetars'][0]], 
			color = colarr[carr == 'magnetars'][0]) #magnetar - 1
		plt.fill_between([2e-1*xconv, 8e3*xconv], 
			[4e33*yconv*karr[carr == 'magnetars'][0], 9e29*yconv*karr[carr == 'magnetars'][0]], 
			[1e36*yconv*karr[carr == 'magnetars'][0], 1e36*yconv*karr[carr == 'magnetars'][0]], 
			color = colarr[carr == 'magnetars'][0]) #magnetar - 2

	if 'ulxs' in classes:
		plt.fill_between([0.6*xconv, 9e3*xconv], 
			[3e38*yconv*karr[carr == 'ulxs'][0], 3e38*yconv*karr[carr == 'ulxs'][0]], 
			[3e40*yconv*karr[carr == 'ulxs'][0], 3e40*yconv*karr[carr == 'ulxs'][0]], 
			color = colarr[carr == 'ulxs'][0]) #ULX

	if 'fbots' in classes:
		plt.fill_between([2.5*xconv, 400*xconv], 
			[1e42*yconv*karr[carr == 'fbots'][0], 1e38*yconv*karr[carr == 'fbots'][0]], 
			[2e44*yconv*karr[carr == 'fbots'][0], 2e44*yconv*karr[carr == 'fbots'][0]], 
			color = colarr[carr == 'fbots'][0]) #FBOT



def obs_dfps(classes = cs, subclasses = sc, subson = False,
			 colors = [defaultc_colors, defaultsc_colors], 
			 xunit = u.d, yunit = u.erg/u.s/u.cm**2, k = 1., ax = plt, **kwargs):

	"""
	Plot the observational DFPS.

	Parameters:
		classes (arr-like): List of classes to plot.
		subclasses (arr-like): If subson = True, plot these subclasses. Should be a nested list, 
			with subclasses grouped in sub-list by class.
		subson (bool): Plot subclasses instead of classes -- if subson = True, must specify subclasses.
		xunit (astropy units object): Units of time for observational DLPS. Default is days.
		yunit (astropy units object): Units of flux for observational DFPS. Default is erg/s/cm**2
		k (float or arr-like): Value of k-correction if plotting outside of 0.3-10 keV band. 
			Should be the length of classes.
		ax (axis object): Axis on which to plot, e.g., ax.plot().
		**kwargs: Modify ax.plot(), see matplotlib documentation.
	"""
	lcs = load()

	xconv = u.d.to(xunit)
	yconv = (u.erg/u.s/u.cm**2).to(yunit)
	if (type(k) == float) or (type(k) == int):
		k = [k]*len(classes)

	if colors == [defaultc_colors, defaultsc_colors]: # hacky test for default parameters (might be temporary if I find a better way)
		if subson:
			colors = colors[1]
		if not subson:
			colors = colors[0]

	if subson:
		for i, c in enumerate(classes):
			ki = k[i]
			if subclasses[i]:
				if sc[i]:
					for k_, s in enumerate(sc[i]):
						col = colors[i][k_]
						for j, obj in enumerate(lcs.loc[(lcs['class'] == c) & (lcs['subclass'] == s)].iterrows()):
							if np.isfinite(obj[1]['z']):
								time = obj[1]['time'] * (1 + obj[1]['z']) * xconv
							if not np.isfinite(obj[1]['z']):
								time = obj[1]['time'] * xconv
							flux = obj[1]['flux'] * yconv * ki
							ax.plot(time, flux, color = col)

							if len(time) == 1:
								ax.scatter(time, flux, color = col) #s = lw^2

			if not subclasses[i]:
				col = colors[i]
				for j, obj in enumerate(lcs.loc[lcs['class'].values == c].iterrows()):
					if np.isfinite(obj[1]['z']):
						time = obj[1]['time'] * (1 + obj[1]['z']) * xconv
					if not np.isfinite(obj[1]['z']):
						time = obj[1]['time'] * xconv
					flux = obj[1]['flux'] * yconv * ki
					ax.plot(time, flux, color = col)

					if len(time) == 1:
						ax.scatter(time, flux, color = col)

	if not subson:
		for i, c in enumerate(classes):
			ki = k[i]
			col = colors[i]
			for j, obj in enumerate(lcs.loc[lcs['class'].values == c].iterrows()):
				if np.isfinite(obj[1]['z']):
					time = obj[1]['time'] * (1 + obj[1]['z']) * xconv
				if not np.isfinite(obj[1]['z']):
					time = obj[1]['time'] * xconv
				flux = obj[1]['flux'] * yconv * ki
				ax.plot(time, flux, color = col)

				if len(time) == 1:
					ax.scatter(time, flux, color = col)

def iso_energy(classes = cs, subclasses = sc, subson = False,
			 colors = [defaultc_colors, defaultsc_colors], 
			 xunit = u.d, yunit = u.erg, k = 1, ax = plt, **kwargs):

	"""
	Plot the isotropic equivalent energy vs. the duration of the light curves.

	Parameters:
		classes (arr-like): List of classes to plot.
		subclasses (arr-like): If subson = True, plot these subclasses. Should be a nested list, 
			with subclasses grouped in sub-list by class.
		subson (bool): Plot subclasses instead of classes -- if subson = True, must specify subclasses.
		xunit (astropy units object): Units of time for observational DLPS. Default is days.
		yunit (astropy units object): Units of flux for observational DFPS. Default is erg/s/cm**2
		k (float or arr-like): Value of k-correction if plotting outside of 0.3-10 keV band. 
			Should be the length of classes.
		ax (axis object): Axis on which to plot, e.g., ax.plot().
		**kwargs: Modify ax.plot(), see matplotlib documentation.
	"""
		

	lcs = load()

	xconv = u.d.to(xunit)
	yconv = (u.erg).to(yunit)
	if (type(k) == float) or (type(k) == int):
		k = [k]*len(classes)

	if colors == [defaultc_colors, defaultsc_colors]: # hacky test for default parameters (might be temporary if I find a better way)
		if subson:
			colors = colors[1]
		if not subson:
			colors = colors[0]

	if subson:
		for i, c in enumerate(classes):
			ki = k[i]
			if subclasses[i]:
				if sc[i]:
					for k_, s in enumerate(sc[i]):
						col = colors[i][k_]
						for j, obj in enumerate(lcs.loc[(lcs['class'] == c) & (lcs['subclass'] == s)].iterrows()):
							for obj in lcs.loc[(lcs['class'] == c) & (lcs['subclass'] == sc)]:
								time = obj[1]['tdur'] * xconv
								eiso = obj[1]['eiso'] * yconv * ki
								ax.scatter(time, eiso, color = col) #s = lw^2

			if not subclasses[i]:
				col = colors[i]
				for j, obj in enumerate(lcs.loc[lcs['class'].values == c].iterrows()):
					time = obj[1]['tdur'] * xconv
					eiso = obj[1]['eiso'] * yconv * ki
					ax.scatter(time, eiso, color = col)

	if not subson:
		for i, c in enumerate(classes):
			ki = k[i]
			col = colors[i]
			for j, obj in enumerate(lcs.loc[lcs['class'].values == c].iterrows()):
				time = obj[1]['tdur'] * xconv
				eiso = obj[1]['eiso'] * yconv * ki
				ax.scatter(time, eiso, color = col)

def lpk_thalf(classes = cs, subclasses = sc, subson = False,
			 colors = [defaultc_colors, defaultsc_colors], 
			 xunit = u.d, yunit = u.erg/u.s, k = 1, ax = plt, **kwargs):

	"""
	Plot the peak luminosity vs. time above 1/2 peak luminosity.

	Parameters:
		classes (arr-like): List of classes to plot.
		subclasses (arr-like): If subson = True, plot these subclasses. Should be a nested list, 
			with subclasses grouped in sub-list by class.
		subson (bool): Plot subclasses instead of classes -- if subson = True, must specify subclasses.
		xunit (astropy units object): Units of time for observational DLPS. Default is days.
		yunit (astropy units object): Units of flux for observational DFPS. Default is erg/s/cm**2
		k (float or arr-like): Value of k-correction if plotting outside of 0.3-10 keV band. 
			Should be the length of classes.
		ax (axis object): Axis on which to plot, e.g., ax.plot().
		**kwargs: Modify ax.plot(), see matplotlib documentation.
	"""

	lcs = load()

	xconv = u.d.to(xunit)
	yconv = (u.erg/u.s).to(yunit)
	if (type(k) == float) or (type(k) == int):
		k = [k]*len(classes)

	if colors == [defaultc_colors, defaultsc_colors]: # hacky test for default parameters (might be temporary if I find a better way)
		if subson:
			colors = colors[1]
		if not subson:
			colors = colors[0]

	if subson:
		for i, c in enumerate(classes):
			ki = k[i]
			if subclasses[i]:
				if sc[i]:
					for k_, s in enumerate(sc[i]):
						col = colors[i][k_]
						for j, obj in enumerate(lcs.loc[(lcs['class'] == c) & (lcs['subclass'] == s)].iterrows()):
							for obj in lcs.loc[(lcs['class'] == c) & (lcs['subclass'] == sc)]:
								time = obj[1]['thalf'] * xconv
								lum = obj[1]['lpk'] * yconv * ki
								ax.scatter(time, lum, color = col) #s = lw^2

			if not subclasses[i]:
				col = colors[i]
				for j, obj in enumerate(lcs.loc[lcs['class'].values == c].iterrows()):
					time = obj[1]['thalf'] * xconv
					lum = obj[1]['lpk'] * yconv * ki
					ax.scatter(time, lum, color = col)

	if not subson:
		for i, c in enumerate(classes):
			ki = k[i]
			col = colors[i]
			for j, obj in enumerate(lcs.loc[lcs['class'].values == c].iterrows()):
				time = obj[1]['thalf'] * xconv
				lum = obj[1]['lpk'] * yconv * ki
				ax.scatter(time, lum, color = col)








	


import numpy as np
import astropy.units as u
import pandas as pd
from scipy.interpolate import interp1d
from sklearn.neighbors import KNeighborsClassifier
from xraydlps.tools import load, convert
import warnings

def warning_on_one_line(message, category, filename, lineno, file=None, line=None):
	return '%s:%s: %s: %s\n' % (filename, lineno, category.__name__, message)

warnings.formatwarning = warning_on_one_line

### internal functions -- callable, but generally not necessary to call explicitly ###

def get_interp(time, lum, t = np.logspace(-8, 4, 100), returns = 'df'):
	"""
	Generate *evenly* sampled light curves for DTW via interpolation.
	
	Run automatically within train_lcmodel.
	
	Parameters:
		time (arr): Time (days) for individual light curve.
		lum (arr): Luminosity (cgs) for individual light curve.
		t (arr): Time (days) used for sampling interpolated light curves.
		returns (str): 'df' to return dataframe for sklearn, 'arr' generates an array
		
	Returns:
		Luminosity (cgs) sampled by t (days) -- dataframe if "df", array if "arr".
	"""
	
	if len(t) >= 10**4:
		warnings.warn('Increased sampling may make prediction slow. Try ~1000 points or fewer.')
		
	if len(time) < 2: #filter for single data points
		if returns == 'arr':
			return np.array([np.nan])
		if returns == 'df':
			lcdict = {}
			for i in range(len(t)):
				lcdict[str(i)] = np.nan
			lcdf = pd.DataFrame(lcdict, index = [0])
			return lcdf
		
	interp = interp1d(time, lum)

	lc = np.zeros_like(t, dtype = float) # so that sklearn will work
	lc[(t >= np.min(time)) & (t <= np.max(time))] = interp(t[(t >= np.min(time)) & (t <= np.max(time))])
	
	if returns == 'arr':
		return lc
	
	if returns == 'df':
		lcdict = {}
		for i in range(len(t)):
			lcdict[str(i)] = lc[i]
		
		lcdf = pd.DataFrame(lcdict, index = [0])
		return lcdf


def dtw_dist(lc1, lc2):
	"""
	Implementation of dynamic time warping for time series classification.

	Only necessary if training a new model.

	Parameters:
		lc1 (series): first light curve Lx
		lc2 (series): second light curve Lx

	Returns:
		Dynamic time warping distance metric for use in classification.
	"""

	n, m = len(lc1), len(lc2)

	dtw = np.full((n + 1, m + 1), np.inf)
	dtw[0, 0] = 0

	for i in range(1, n + 1):
		for j in range (1, m + 1):
			cost = abs(lc1[i - 1] - lc2[j - 1]) 
			dtw[i, j] = cost + np.min([dtw[i - 1, j], dtw[i, j - 1], dtw[i - 1, j - 1]])

	return dtw[n, m]


##########
# * * * * 
##########


def lc_class(time, lum, tunits = u.d, lunits = u.erg/u.s, k = 1, verbose = True, skip_zero = True, full = False):
	"""
	Classify light curve based on default pre-trained KNN using dynamic time warping on time-series data.

	At best ~70% accurate based on testing for correct *primary* classification. Use with caution!

	Parameters:
		time (arr): Input luminosity data for light curve. Default unit in days.
		lum (arr): Input luminosity data for light curve. Default unit in erg/s.
		tunits (astropy units object): Input units of time.
		lunits (astropy units object): Input units of luminosity.
		k (float): K-correction to 0.3 - 10 keV band if input light curve in different band.
		verbose (bool): If verbose, print all possible classes + classification probability.
		skip_zero (bool): If verbose and skip_zero = True, only show classes with probability > 0.
		full (bool): Use model trained on full available dataset rather than 70/30 training/test split.
			Accuracy comes from testing model used when full = False. Ultimate accuracy may vary.

	Returns:
		If verbose, returns class + classification probability. If not verbose, returns class.
	"""

	warnings.warn("Model is ~70\% accurate based on testing. Use with caution!!")

	if not full:
		mod = load('lc_mod.pkl')
	if full:
		mod = load('lc_mod_full.pkl')
	tconv = tunits.to(u.d)
	lconv = k*lunits.to(u.erg/u.s)
	lx = get_interp(time*tconv, lum*lconv)

	if not verbose:
		out = mod.predict(lx)
		print(out[0])
		return out

	if verbose:
		labels_ = mod.classes_

		out_ = mod.predict_proba(lx)

		out = np.sort(out_)[::-1]
		labels = labels_[np.argsort(out_)][::-1]

		if not skip_zero:
			df = pd.DataFrame.from_dict({'class': labels, 'classification probability':out})

		if skip_zero:
			df = pd.DataFrame.from_dict({'class': labels[out > 0], 'classification probability':out[out > 0]})

		print(df.to_string(index=False))
		return labels_, out_



def sum_class(time, lum, tunits = u.d, lunits = u.erg/u.s, k = 1, verbose = True, skip_zero = True, full = False):
	"""
	Classify light curve based on default pre-trained KNN using summary values.

	At best ~60% accurate based on testing for correct *primary* classification. Use with caution!!

	If you would like to train your own model, use xraydlps.classify.train_summodel().

	Parameters:
		time (arr): Input luminosity data for light curve. Default unit in days.
		lum (arr): Input luminosity data for light curve. Default unit in erg/s.
		tunits (astropy units object): Input units of time.
		lunits (astropy units object): Input units of luminosity.
		k (float): K-correction to 0.3 - 10 keV band if input light curve in different band.
		verbose (bool): If verbose, print all possible classes + classification probability.
		skip_zero (bool): If verbose and skip_zero = True, only show classes with probability > 0.
		full (bool): Use model trained on full available dataset rather than 70/30 training/test split.
			Accuracy comes from testing model used when full = False. Ultimate accuracy may vary.

	Returns:
		If verbose, returns class + classification probability. If not verbose, returns class.
	"""

	warnings.warn("Model is ~60\% accurate based on testing. Use with caution!!")

	if not full:
		mod = load('sum_mod.pkl')
	if full:
		mod = load('sum_mod_full.pkl')
	tconv = tunits.to(u.d)
	lconv = k*lunits.to(u.erg/u.s)
	lpk, thalf, eiso, dur = convert(time*tconv, lum*lconv)

	pred = {'lpk':lpk, 'eiso':eiso, 'tdur':dur}
	# preddf = pd.DataFrame.from_dict(pred) 
	preddf = pd.DataFrame(pred, index = [0]) 

	if not verbose:
		out = mod.predict(preddf)
		print(out[0])
		return out

	if verbose:
		labels_ = mod.classes_

		out_ = mod.predict_proba(preddf)

		out = np.sort(out_)[::-1]
		labels = labels_[np.argsort(out_)][::-1]

		if not skip_zero:
			df = pd.DataFrame.from_dict({'class': labels, 'classification probability':out})
		if skip_zero:
			df = pd.DataFrame.from_dict({'class': labels[out > 0], 'classification probability':out[out > 0]})

		print(df.to_string(index=False))
		return labels_, out_



def train_lcmodel(dataframe = load(), n_neighbors = 1, include_singles = True, frb_as_magnetar = False, 
				verbose = True, t = np.logspace(-8, 4, 100), weights = 'uniform', test = 0.3, train = 0.7, **kwargs):
	"""
	Train model that classifies based on matching light curves.
	
	Defaults are representative of training for models in xraydlps.classify.lc_class.
	
	Parameters:
		dataframe (df): Dataframe that contains data and labels. Labels column 'class', data column 'lc'.
		n_neighbors (int): Number of neighbors used by KNN. If include_singles, n_neighbors = 1.
		include_singles (bool): Include SBO (SN2008D) and FRB (FRB200428) in classifier.
		frb_as_magnetar (bool): If include_singles = False, re-assign FRB to magnetar label. Otherwise ignored.
		verbose (bool): If verbose, print information about training and return test set.
		t (arr): Time (days) used for sampling interpolated light curves.
		weights (str): 'uniform' or 'distance' -- weights used when k neighbors > 1.
		test (float): Fraction of sample to be allocated to test set.
		train (float): Fraction of sample to be allocated to training set.
		**kwargs: Other keyword arguments for sklearn.neighbors.KNeighborsClassifier().
		
	Returns:
		Trained model to classify input light curve.
		
		If verbose, also return the training and test sets (data, labels) - dtrain, dtest, ltrain, ltest. 
	"""
	dataframe['lc'] = [get_interp(i[1].time, i[1].lum, t = t) for i in dataframe.iterrows()]
	
	labels = [] #Labels to be used for classification.
	data = [] #Consistently sampled light curve data for KNN with DTW.
	for i in dataframe.iterrows():
		if len(i[1].lc) > 1: #filter for light curves of length 1
			labels.append(i[1]['class'])
			data.append(i[1].lc)
			
	labels = np.array(labels)
	data = np.array(data)
	
	if include_singles:
		if n_neighbors != 1:
			warnings.warn("Warning: Since include_singles = True, setting n_neighbors = 1.")
		n_neighbors = 1 #since single light curve for each of SBO and FRB
		
		## split test-train so that train includes SBO and FRB
		labels_ = labels[(labels != 'sbo') & (labels != 'frb')]
		data_ = data[(labels != 'sbo') & (labels != 'frb')]
		
		if (test > 0) and (train < 1):
			dtrain, dtest, ltrain, ltest = train_test_split(data_, labels_, test_size = test, train_size = train)

			dtrain = np.concatenate((dtrain, data[(labels == 'sbo') | (labels == 'frb')]))
			ltrain = np.concatenate((ltrain, labels[(labels == 'sbo') | (labels == 'frb')]))
		
		
	if not include_singles:
		if frb_as_magnetar:
			labels[labels == 'frb'] = 'magnetars'
			
		## filter to skip SBO and FRB from test + training
		labels_ = labels[(labels != 'sbo') & (labels != 'frb')]
		data_ = data[(labels != 'sbo') & (labels != 'frb')]
		
		if (test > 0) and (train < 1):
			dtrain, dtest, ltrain, ltest = train_test_split(data_, labels_, test_size = 0.3, train_size = 0.7)
	
	knn = KNeighborsClassifier(metric = dtw_dist, n_neighbors = n_neighbors, weights = weights, **kwargs)
	
	if (test > 0) and (train < 1):
		if not verbose:
			return knn.fit(X = dtrain, y = ltrain)
		if verbose:
			return knn.fit(X = dtrain, y = ltrain), dtrain, dtest, ltrain, ltest
	
	if (test == 0) and (train == 1):
		if not verbose:
			return knn.fit(X = data_, y = labels_)
		if verbose:
			return knn.fit(X = data_, y = labels_), data_, [], labels_, []
	
def train_summodel(dataframe = load(), n_neighbors = 1, use_thalf = False, min_len = 1, 
				   include_singles = True, frb_as_magnetar = False, verbose = True, 
				   weights = 'uniform', p = 1, test = 0.3, train = 0.7, **kwargs):
	"""
	Train model that classifies based on matching summary statistics.
	
	Defaults are representative of training for models in xraydlps.classify.sum_class.
	
	Parameters:
		dataframe (df): Dataframe that contains data and labels. Labels column 'class', data column 'lc'.
		n_neighbors (int): Number of neighbors used by KNN. If include_singles, n_neighbors = 1.
		use_thalf (bool): If True, include thalf -- time above 1/2 Lpk -- in classification. Requires len(lc) > 1.
		min_len (int): Minimum length of included light curve -- if use_thalf = True, min_len becomes 2.
		include_singles (bool): Include SBO (SN2008D) and FRB (FRB200428) in classifier.
		frb_as_magnetar (bool): If include_singles = False, re-assign FRB to magnetar label. Otherwise ignored.
		verbose (bool): If verbose, return training and test sets.
		weights (str): 'uniform' or 'distance' -- weights used when k neighbors > 1.
		p (int): 1 or 2 -- determines power parameter for Minkowski distance metric.
		test (float): Fraction of sample to be allocated to test set.
		train (float): Fraction of sample to be allocated to training set.
		**kwargs: Other keyword arguments for sklearn.neighbors.KNeighborsClassifier().
		
	Returns:
		Trained model to classify input light curve via summary statistics.
		
		If verbose, also return the training and test sets (data, labels) - drain, dtest, ltrain, ltest.
	"""
	
	if not use_thalf:
		data = dataframe[['lpk', 'eiso', 'tdur']].copy()
		labels = dataframe[['class']].copy()
		
		data = data.loc[dataframe.time.map(len) >= min_len]
		labels = labels.loc[dataframe.time.map(len) >= min_len]
	
	if use_thalf:
		warnings.warn("Use caution; thalf is a less robust derived quantity dependent on time-sampling and other factors.")
		data = dataframe[['lpk', 'thalf', 'eiso', 'tdur']].copy()
		labels = dataframe[['class']].copy()
		
		data = data.loc[(dataframe.time.map(len) >= min_len) & (dataframe.thalf.notnull())]
		labels = labels.loc[(dataframe.time.map(len) >= min_len) & (dataframe.thalf.notnull())]
	
	
	if include_singles:
		if n_neighbors != 1:
			warnings.warn("Warning: Since include_singles = True, setting n_neighbors = 1.")
		n_neighbors = 1 #since single light curve for each of SBO and FRB
		
		## split test-train so that train includes SBO and FRB
		labels_ = labels.loc[(labels['class'] != 'sbo') & (labels['class'] != 'frb')]
		data_ = data.loc[(labels['class'] != 'sbo') & (labels['class'] != 'frb')]
		
		if (test > 0) and (train < 1):
			dtrain, dtest, ltrain, ltest = train_test_split(data_, labels_, test_size = test, train_size = train)

			dtrain = dtrain.append(data.loc[(labels['class'] == 'sbo') | (labels['class'] == 'frb')])
			ltrain = ltrain.append(labels.loc[(labels['class'] == 'sbo') | (labels['class'] == 'frb')])

		
	if not include_singles:
		if frb_as_magnetar:
			labels[labels == 'frb'] = 'magnetars'
			
		## filter to skip SBO and FRB from test + training
		labels_ = labels.loc[(labels['class'] != 'sbo') & (labels['class'] != 'frb')]
		data_ = data.loc[(labels['class'] != 'sbo') & (labels['class'] != 'frb')]
		
		if (test > 0) and (train < 1):
			dtrain, dtest, ltrain, ltest = train_test_split(data_, labels_, test_size = test, train_size = train)
	
	knn = KNeighborsClassifier(n_neighbors = n_neighbors, weights = weights, p = p, **kwargs)
	
	if (test > 0) and (train < 1):
		if not verbose: 
			return knn.fit(X = dtrain, y = ltrain)
		if verbose:
			return knn.fit(X = dtrain, y = ltrain), dtrain, dtest, ltrain, ltest
		
	if (test == 0) and (train == 1):
		if not verbose:
			return knn.fit(X = data_, y = labels_)
		if verbose:
			return knn.fit(X = data_, y = labels_), data_, [], labels_, []


import pickle
import astropy.units as u
from astropy.cosmology import FlatLambdaCDM, z_at_value
import astropy.constants as c
import numpy as np
from scipy.interpolate import interp1d
from sklearn.metrics import auc

# import pkgutil
import pkg_resources
import pandas as pd

cosmo = FlatLambdaCDM(H0 = 69.6, Om0 = 0.286)
sky = (4*np.pi*u.radian**2).to(u.deg**2)

def get_z(dl):
	return z_at_value(cosmo.luminosity_distance, dl*u.Mpc, zmax = 1e6)

def get_dcom(dl):
	## bit of a roundabout way to get comoving distance, but assuming luminosity distances as input
	return cosmo.comoving_distance(get_z(dl)).value

DATA_PATH = pkg_resources.resource_filename('xraydlps', 'data/')

##########
# * * * * 
##########

def load(pkl_file = 'lcs.pkl'):
	"""
	Convenience function to load pickled files. User-facing application is to access lcs.pkl.

	Parameters:
		pkl_file (str): Path to pickled file.

	Returns:
		Unpickled file.
	"""
	pkl_path = pkg_resources.resource_filename('xraydlps', 'data/'+pkl_file)

	with open(pkl_path, 'rb') as pkl:
		pk = pickle.load(pkl)
	return pk


def n_obs(lpk, rate, sens, fov, 
	lunits = u.erg/u.s, runits = u.Gpc**-3/u.yr, 
	funits = u.erg/u.s/u.cm**2, fovunits = u.deg**2, perunits = u.yr):
	"""
	Return anticipated detection/observation rate for particular class of transients and instrument.

	Parameters:
		lpk (float): Peak x-ray magnitude of transient class. Default units erg/s.
		rate (float): Volumetric event rate for transient class. Default units Gpc**-3/yr.
		sens (float): Flux limit of instrument. Default units erg/s/cm**2.
		fov (float): Instrument field of view. Default units deg**2.

		lunits (astropy units object): Input units of lpk.
		runits (astropy units object): Input units of rate.
		funits (astropy units object): Input units of sens.
		fovunits (astropy units object): Input units of fov.
		perunits (astropy units object): Sets unit of time for returned Nobs. Default is Nobs/yr.

	Returns:
		n_obs (float): N_obs/yr (by default) or N_obs/perunits (if specified).

	"""

	lconv = lunits.to(u.erg/u.s)
	rconv = runits.to(u.Gpc**-3/u.yr)
	fconv = funits.to(u.erg/u.s/u.cm**2)
	fovconv = fovunits.to(u.deg**2)
	outconv = (1/u.yr).to(1/perunits)


	frac = (fov*fovconv)/sky.value
	depth = np.sqrt((lpk*lconv)/(4*np.pi*sens*fconv))*u.cm.to(u.Mpc)
	if depth < 6701.2: #condition on z <= 1
		inst_depth = depth 
	else:
		inst_depth = 6701.2 #set z = 1 (Mpc, luminosity distance)

	inst_depth_com = get_dcom(inst_depth)
	inst_vol = 4/3*np.pi*(inst_depth_com*u.Mpc.to(u.Gpc))**3 * frac

	nobs = rate*rconv*inst_vol*outconv

	return nobs


def convert(time, lum, k = 1, tunits = u.d, lunits = u.erg/u.s):
	"""
	Generate Lpk, t_half, E_iso, dur points from light curve for plotting and classification.

	Parameters:
		time (arr-like): Input time data for light curve. Default units in days.
		lum (arr-like): Input luminosity data for light curve. Default unit in erg/s.
		k (float): K-correction from input band --> 0.3-10 keV.
		tunits (astropy units object): Input units of time.
		lunits (astropy units object): Input units of lum.

	Returns:
		lpk, thalf, eiso, dur: Peak luminosity, duration above 0.5*Lpk, isotropic equivalent
				energy, and duration. Output in erg/s, days, erg, days.

	** Note that light curves must have multiple data points for this to work properly. **
	"""

	tconv = tunits.to(u.d)
	lconv = k*lunits.to(u.erg/u.s)

	time = tconv * time
	lum = k * lconv * lum

	lpk = np.max(lum)
	dur = np.max(time)

	if len(time) < 2:
		eiso = time*lum
		thalf = np.nan

	if len(time) >= 2:
		eiso = auc(time, lum)
		time_check = np.linspace(np.min(time), np.max(time), 1000)
		check = interp1d(time, lum)
		thalf = (np.max(time) - np.min(time))*len(check(time_check)[check(time_check) >= 0.5*lpk])/1000

	return lpk, thalf, eiso, dur



def get_stat(c, sc = None, stat = 'max', axis = 'lum'):
    """
    Return basic stats (min, max, mean, median) about (sub)classes of transient.
    
    Hint: use xraydlps.tools.list_classes() to see available (sub)classes.
    
    Parameters:
        c (str): Class for which to compute the statistic.
        sc (str): Subclass for which to compute statistic, if applicable.
        stat (str): Statistic to computea across light curves -- 
            options are 'min', 'max', 'mean', and 'median'.
        axis (str): Axis over which to compute statistic -- 
            options are 'time', 'lum', 'flux', lpk', 'thalf', 'eiso', and 'tdur'
            
    Returns:
        Statistic across all light curves in a given class.
        If time, units are in days; lum, units are erg/s; flux, erg/s/cm2;
        	lpk, erg/s; thalf, days; eiso, erg; tdur, days.
    """
    lcs = load()
    
    stat_out = []
    if not sc:
        for i in lcs.loc[lcs['class'] == c].iterrows():
            stat_out.append(i[1][axis])
    if sc:
        for i in lcs.loc[(lcs['class'] == c) & (lcs['subclass'] == sc)].iterrows():
            stat_out.append(i[1][axis])
    
    
    if stat == 'min':
        return np.nanmin(np.concatenate(stat_out).flatten())
    
    if stat == 'max':
        return np.nanmax(np.concatenate(stat_out).flatten())
    
    if stat == 'mean':
        return np.nanmean(np.concatenate(stat_out).flatten())
    
    if stat == 'median':
        return np.nanmedian(np.concatenate(stat_out).flatten())


import setuptools

setuptools.setup(
	name = "xraydlps",
	version = "1.0.1",
	author = "Ava Polzin",
	author_email = "apolzin@uchicago.edu",
	description = "Tools to help plot the X-ray duration-luminosity phase space and use it for classification.",
	packages = ["xraydlps", "xraydlps/plot", "xraydlps/classify", "xraydlps/tools"],
	url = "https://github.com/avapolzin/X-rayLCs",
	license = "MIT",
	classifiers = [
		"Development Status :: 5 - Production/Stable",
		"Intended Audience :: Science/Research",
		"License :: OSI Approved :: MIT License",
		"Programming Language :: Python",
		"Topic :: Scientific/Engineering :: Astronomy",
		"Topic :: Scientific/Engineering :: Physics"
		],
	python_requires = ">3",
	install_requires = ["numpy", "matplotlib", "pandas", "scikit-learn", "astropy", "scipy"],
	package_data={'xraydlps': ['data/*']}
	)
